
;{ ----- File description Header : WinFX - Library -----

;======================================================================================================================
;
; Main-Project Name .:  WinFX - Library
; Project Mainfile ..:  Y:\EDV\Programmierung\PureBasic\Projekte\Library - WinFX\WinFX-Library.pbp
; Files in Project ..:  1/1/0/0/0/0/0/0/0 - (pb/pbi/userlibs/dll/images/sound/ini/data/database)
; Proj. Description .:  Library which supplies Functions, to add additional Effects to "standard" PB-Windows
;
; This Files Name....:  Lib_WinFX.pbi
; Location ..........:  Y:\EDV\Programmierung\PureBasic\Projekte\Includes\
; Made public .......:  yes
; Published URL .....:  http://www.bmi-online.de/public/PB-Sources/includes/Lib_WinFX.pbi
; Description .......:  This file contains all necessary parts of the WinFX-Library (1/1)
;
;                       Needed   Includes ...............: 0
;                       Declared Macros .................: 0
;                       Defined  Constants ..............: 2
;                       Defined  Variables (local/gobal) : 1 / 0
;                       Defined  LLists    (local/gobal) : 0 / 0
;                       Defined  Arrays    (local/gobal) : 0 / 0
;                       Defined  Maps      (local/gobal) : 0 / 0

;                       Declared Procedures: WinFX_HideFromTaskBar      (WinNr, State = #True)
;                                            WinFX_TurnOffLayer         (WinNr)
;                                            WinFX_ClickThrough         (WinNr, State = #True)
;                                            WinFX_Transparency         (WinNr, Level, SkinningColor = #WinFX_NoScinning)
;                                            WinFX_GetTaskBarState      (WinNr)
;                                            WinFX_GetLayerState        (WinNr)
;                                            WinFX_GetClickThroughState (WinNr)
;
; additional Infos ..:  Some WinFX-Functions will turn the Window in a - so called - "layered Mode"
;                       If a Window is in that state, it may not be seen on some ScreenShots (depends on the Method which is used to make the ScreenShot).
;                       Further a layered Window could slow down the Screenrefesh of the Desktop (sometimes dramatically on old systems with slow Grafikcards).
;                       The Layered-Mode of a Window will be switched of automatically by the WinFX-Functions, if it's not needed anymore.
;                       (special behavior for WinFX_Transparency() - see Parameter: Level=-1 for additional info)
;                       To turn a window back to "normal" mode by yourself, please see WinFX_TurnOffLayer()
;
; PB-Version .............:  PB 4.51  ->  x86 - tested   |   x64 - not yet tested
; supported Functionality :  x86            - yes
;                            x64            - not tested, should work
;                            EnableExplicit - yes
;                            Threadsave     - not tested, should work
;                            Unicode        - not tested, should work
;                          
; Supported Platform .....:  Windows (should work on all Windows-Versions from Win2000 to Win7)
; uses OS-API-calls ......:  yes
; Tested on Systems ......:  WinXP-32Bit
;
; Copyrights .............:  Sansetsu Deutschland GmbH
; Primarily Author(s) ....:  Albert Cremers (Luke)
; Licensed as OpenSource .:  yes
; Creation Date ..........:  03.10.2010
; Date, last Modification :  06.10.2010
; Version ................:  1.3.2.0
; Modification(s) by .....:  
; Descr. of Modifications :  added Functions: WinFX_GetTaskBarState(), WinFX_GetLayerState(), WinFX_GetClickThroughState() (Luke)
;                            fixed some spelling mistakes in description (more to come)  ;)
;                            Added skinning-function and SkinningColor-Parameter to WinFX_Transparency() and new Constant: #WinFX_NoSkinning
; Remarks ................:  
;}=====================================================================================================================

CompilerIf Defined(INCLUDE_WinFXLib, #PB_Constant) = #False
;{ ----- Constants
	#INCLUDE_WinFXLib = #True
	#WinFX_NoSkinning = -1
;}
;{ ----- Variables
	Define WinFX_EXStyleState = 0
;}
;{ ----- Procedures
Procedure	WinFX_HideFromTaskBar(WinNr, State = #True, AutoUnHide = #True)
	;{ ----- Proc-Description Header
	; =============================================================================================
	; 	Procedure Name ....:  WinFX_HideFromTaskBar()
	; 	Description .......:  Hides a window from the Taskbar, or brings it back on.
	;
	; 	Syntax ............:  Procedure WinFX_HideFromTaskBar(WinNr, State = #True, AutoUnHide = #True)
	; 	Parameter(s) ......:  WinNr .... - The PB Window-Number of a Window
	; 	                      State .... = #True (default value) - the Window will be hidden from the Taskbar.
	; 	                                   #False - the Window will be brought back to the Taskbar.
	;                        AutoUnHide = #True (default value) - the window will be unhidden at the end (see "additional Info" for more details)
	;                                   = #False - the Window will not be unhidden at the end, so after calling this function, the window will be hidden (invisible).
	;  additional Info ...:  To enable the WinFX_HideFromTaskBar()-effect, the window has to be invisible (in hidden mode).
	;                        So WinFX_HideFromTaskBar() will auto-hide the window, before it sets this effect.
	;                        After the effect is set, the window will be unhidden by default (see the "AutoUnHide"-Parameter for more details)
	;                        So, if you not want to let your window become visible, make sure to set the "AutoUnHide"-Parameter to #False.
	;
	; 	Return value(s) ...:  Nothing
	; 	
	; 	Copyrights .............:  Sansetsu Deutschland GmbH
	; 	Primarily Author(s) ....:  Albert Cremers (Luke)
	; 	Licensed As OpenSource .:  yes
	; 	Date, created ..........:  03.10.2010
	; 	Date, last modified ....:  04.10.2010
	; 	Version ................:  1.0.3.1
	; 	Modification(s) by .....:  
	; 	Descr. of Modifications :  added "AutoUnHide" parameter (Luke)
	; 	Remarks ................:  
	;} ==============================================================================
	
	If IsWindow(WinNr)
		HideWindow(WinNr,#True)
		If State
			SetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE,GetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE) | #WS_EX_TOOLWINDOW)
		Else
			SetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE,GetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE) & ~#WS_EX_TOOLWINDOW)
		EndIf
		If AutoUnHide
			HideWindow(WinNr,#False)
		EndIf
	EndIf
EndProcedure
Procedure	WinFX_TurnOffLayer(WinNr)
	;{ ----- Proc-Description Header
	; =============================================================================================
	; 	Procedure Name ....:  WinFX_TurnOffLayer()
	; 	Description .......:  Turns off the "layered Mode" of the window and switches it back to "normal" mode.
	;                        This turns off any WinFX-Effect that will need the window to be in "layered Mode".
	;                        The following functions are affected by this: WinFX_ClickThrough()
	;                                                                      WinFX_Transparency()
	;                        Note: For more informations about the "layered-Mode", see "additional Infos" in the "File description Header" above.
	;
	; 	Syntax ............:  WinFX_TurnOffLayer(WinNr)
	; 	Parameter(s) ......:  WinNr  - The PB Window-Number of a Window
	; 	Return value(s) ...:  Nothing
	; 	
	; 	Copyrights .............:  Sansetsu Deutschland GmbH
	; 	Primarily Author(s) ....:  Albert Cremers (Luke)
	; 	Licensed As OpenSource .:  yes
	; 	Date, created ..........:  03.10.2010
	; 	Date, last modified ....:  03.10.2010
	; 	Version ................:  1.0.0.0
	; 	Modification(s) by .....:  
	; 	Descr. of Modifications :
	; 	Remarks ................:  
	;} ==============================================================================
	
	Shared  WinFX_EXStyleState
	If IsWindow(WinNr)
		WinFX_EXStyleState = 0
		SetLayeredWindowAttributes_(WindowID(WinNr), #Null, #Null, #LWA_ALPHA)
		SetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE,GetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE) & ~#WS_EX_LAYERED  & ~#WS_EX_TRANSPARENT)
	EndIf
EndProcedure
Procedure	WinFX_ClickThrough(WinNr, State = #True)
	;{ ----- Proc-Description Header
	; =============================================================================================
	; 	Procedure Name ....:  WinFX_ClickThrough()
	; 	Description .......:  turns the window into "click-through" Mode
	; 	                      In the "click-through" Mode all Mouse-Events (mouse-movement, clicks, hoover, ...) will go right through the Window to the window underneath.
	;                        That means, that the window will not receive any mouse-events from now on.
	;                        So keep that in mind, when you turn a window in "click-through" Mode and include a facility to close the window and/or react on userinput, rather than by Window-Events.
	;                        Note: For this effect, the window will be turned into "layered Mode".
	;                              For more details and limitations on that, see "additional Infos" in the "File description Header" above.
	;
	; 	Syntax ............:  WinFX_ClickThrough(WinNr, State = #True)
	; 	Parameter(s) ......:  WinNr  - The PB Window-Number of a Window
	; 	                      State  - With the State, you can turn the "click-through" Mode on and off.
	; 	                               #True (or any other value other than 0) - will turn the Effect on.
	; 	                               #False - will turn the Effect off.
	; 	                               The Default-Value is: #True
	; 	Return value(s) ...:  Nothing
	; 	
	; 	Copyrights .............:  Sansetsu Deutschland GmbH
	; 	Primarily Author(s) ....:  Albert Cremers (Luke)
	; 	Licensed As OpenSource .:  yes
	; 	Date, created ..........:  03.10.2010
	; 	Date, last modified ....:  04.10.2010
	; 	Version ................:  1.0.3.1
	; 	Modification(s) by .....:  
	; 	Descr. of Modifications :  
	; 	Remarks ................:  
	;} ==============================================================================
	
	Shared WinFX_EXStyleState
	If IsWindow(WinNr)
		If State
			WinFX_EXStyleState = WinFX_EXStyleState | #WS_EX_TRANSPARENT
			SetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE,GetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE) | #WS_EX_LAYERED | #WS_EX_TRANSPARENT)
			If Not WinFX_EXStyleState & #WS_EX_LAYERED
				SetLayeredWindowAttributes_(WindowID(WinNr), #Null, 255, #LWA_ALPHA)
			EndIf
		Else
			SetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE,GetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE) & ~#WS_EX_TRANSPARENT)
			WinFX_EXStyleState = WinFX_EXStyleState & ~#WS_EX_TRANSPARENT
			If Not WinFX_EXStyleState
				WinFX_TurnOffLayer(WinNr)
			EndIf
		EndIf
	EndIf
EndProcedure
Procedure	WinFX_Transparency(WinNr, Level, SkinningColor = #WinFX_NoSkinning)
	;{ ----- Proc-Description Header
	; =============================================================================================
	; 	Procedure Name ....:  WinFX_Transparency()
	; 	Description .......:  Turns on the transparency effect for a window and sets its opacity to the given "Level"
	;                        If you set the "SkinningColor", this Color will be used to set an AlphaKey on that window, so it will be transparent on all Pixels which will have the given Color.
	;                        Note: For this effect, the window will be turned into "layered Mode".
	;                              For more details and limitations on that, see "additional Infos" in the "File description Header" above.
	; 	
	; 	Syntax ............:  WinFX_Transparency(WinNr, Level)
	; 	Parameter(s) ......:  WinNr  - The PB Window-Number of a Window
	; 	                      Level  - The transparency-level To be set
	; 	                               Level could be -1 or within the ranges from -1 0-100
	; 	                               Where 0 will result in a not transparent window, while 100 will result in a fully transparent window
	; 	                               Even If you set the transparency Level to 0, the Window will stay in layered mode (even though it has no transparency at all on Level 0)
	; 	                               -1 : will set the transparency Level To 0, will disable skinning as well and will TURN OFF the layered mode (if it is not used by any other WinFX-Effect)
	;                        SkinningColor - enables the skinning function
	;                                 by setting a SkinningColor, this color will be used as an AlphaKey to skin the window.
	;                                 That means, that the Pixels of a Window will be totally transparent, which fits the given SkinningColor.
	;                                 On these (transparent) Pixels, all Events will pathed through the beneath window, eventhough the "Click-Through" Mode is not aktivated.
	;                                 Note: If you plan to skin a Window, it's recommended to use a borderless Window.
	; 	Return value(s) ...:  Nothing
	; 	
	; 	Copyrights .............:  Sansetsu Deutschland GmbH
	; 	Primarily Author(s) ....:  Albert Cremers (Luke)
	; 	Licensed As OpenSource .:  yes
	; 	Date, created ..........:  03.10.2010
	; 	Date, last modified ....:  06.10.2010
	; 	Version ................:  1.2.0.1
	; 	Modification(s) by .....:  
	; 	Descr. of Modifications :  
	; 	Remarks ................:  
	;} ==============================================================================
	
	Shared WinFX_EXStyleState
	If IsWindow(WinNr) And level >= -1 And level < 101
		
		If Level < 0
			WinFX_EXStyleState = WinFX_EXStyleState & ~#WS_EX_LAYERED
			Level = 0
			If Not WinFX_EXStyleState
				WinFX_TurnOffLayer(WinNr)
			EndIf
		Else
			If Not WinFX_EXStyleState 
				SetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE,GetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE) | #WS_EX_LAYERED)
			EndIf
			WinFX_EXStyleState = WinFX_EXStyleState | #WS_EX_LAYERED
		EndIf
		If SkinningColor = #WinFX_NoSkinning
			SetLayeredWindowAttributes_(WindowID(WinNr), #Null, 255*(100-level)/100, #LWA_ALPHA)
		Else
			SetLayeredWindowAttributes_(WindowID(WinNr), SkinningColor, 255*(100-level)/100, #LWA_ALPHA | #LWA_COLORKEY)
		EndIf
	EndIf 
EndProcedure
Procedure	WinFX_GetTaskBarState(WinNr)
	; Returns #True, if Window is hidden from Taskbar
	If IsWindow(WinNr)
		ProcedureReturn ((GetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE) & #WS_EX_TOOLWINDOW) Or #False)		; Will return #True or False instead of a value
	EndIf
EndProcedure
Procedure	WinFX_GetLayerState(WinNr)
	; Returns #True, if Window is in "layered Mode"
	If IsWindow(WinNr)
		ProcedureReturn ((GetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE) & #WS_EX_LAYERED) Or #False)			; Will return #True or False instead of a value
	EndIf
EndProcedure
Procedure	WinFX_GetClickThroughState(WinNr)
	; Returns #True, if Window is in "click-through Mode"
	If IsWindow(WinNr)
 		ProcedureReturn ((GetWindowLong_(WindowID(WinNr),#GWL_EXSTYLE) & #WS_EX_TRANSPARENT) Or #False)		; Will return #True or False instead of a value
	EndIf
EndProcedure
;}
CompilerEndIf
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 242
; FirstLine = 41
; Folding = LQ5
; EnableXP